from django.core.files import File
from django.core.files.base import ContentFile
from swampdragon.serializers.field_deserializers import BaseFieldDeserializer, register_field_deserializer
from swampdragon_upload.file_upload_handler import get_file_location


class FileDeserializer(BaseFieldDeserializer):
    def __call__(self, model_instance, key, val):
        if not val:
            return
        if isinstance(val, str):
            return
        if isinstance(val, list):
            val = val[0]
        file_id = int(val['file_id'])
        if not file_id > 0:
            return
        path = get_file_location(val['file_name'], val['file_id'])
        uploaded_file = File(open(path, 'rb'))
        setattr(model_instance, key, val['file_name'])
        getattr(model_instance, key).save(
            val['file_name'],
            ContentFile(uploaded_file.read()),
            save=False
        )


class ImageDeserializer(FileDeserializer):
    pass


register_field_deserializer('FileField', FileDeserializer)
register_field_deserializer('ImageField', ImageDeserializer)



from os.path import join
from os import makedirs
from django.conf import settings
from datetime import datetime
import time


def make_file_id(file_data):
    timestamp = datetime.now()
    timestamp = time.mktime(timestamp.timetuple()) * 1e3 + timestamp.microsecond / 1e3
    timestamp = '{}'.format(timestamp).encode()
    return str(abs(hash(file_data + timestamp)))


def get_file_location(file_name, file_id):
    path = join(settings.MEDIA_ROOT, 'tmp')
    path = join(path, str(file_id))
    try:
        makedirs(path)
    except:
        pass
    return join(path, file_name)


def get_file_url(file_name, file_id):
    path = join(settings.MEDIA_URL, 'tmp')
    path = join(path, str(file_id))
    return join(path, file_name)



import json
from tornado.web import RequestHandler
from swampdragon_upload.file_upload_handler import make_file_id, get_file_location, get_file_url


class FileUploadHandler(RequestHandler):
    @classmethod
    def get_name(cls):
        return cls.route_name

    def _set_access_control(self):
        origin = self.request.headers['origin']
        orig_test = origin.split('/')[-1]
        if ':' in orig_test:
            orig_test = orig_test.split(':')[0]
        if not self.request.host.split(':')[0] == orig_test:
            return
        self.set_header('Access-Control-Allow-Credentials', True)
        self.set_header('Access-Control-Allow-Methods', 'POST')
        self.set_header('Access-Control-Allow-Origin', origin)

    def get(self, *args, **kwargs):
        self.write('Hello!')

    def post(self, *args, **kwargs):
        self._set_access_control()
        files = self.request.files['file']
        response = {'files': []}
        for f in files:
            file_id = make_file_id(f['body'])
            file_name = f['filename']
            named_file = open(get_file_location(file_name, file_id), 'wb')
            named_file.write(f['body'])
            named_file.close()
            response['files'].append({
                'file_id': file_id,
                'file_name': file_name,
                'file_url': get_file_url(file_name, file_id)
            })
        self.write(json.dumps(response))

    def options(self, *args, **kwargs):
        self._set_access_control()

    def file_upload(self, request):
        pass
